@@ -10,7 +10,7 @@
 #'@param vinv The V inverse matrix
 #'@return The effect power for the parameters
 #'@keywords internal
-effectpower = function(RunMatrix, levelvector, anticoef, alpha, vinv = NULL) {
+effectpower = function(RunMatrix, levelvector, anticoef, alpha, vinv = NULL, degrees=NULL) {
 
   L = replicate(length(levelvector), matrix(NA, nrow = 0, ncol = 0))
 
@@ -19,11 +19,19 @@ effectpower = function(RunMatrix, levelvector, anticoef, alpha, vinv = NULL) {
   for (i in 1:(length(g) - 1)) {
     L[[i]] = genparammatrix(dim(attr(RunMatrix, "modelmatrix"))[2], levelvector[i], g[i])
   }
-
+  if (is.null(degrees)) {
+    degrees = rep(dim(attr(RunMatrix, "modelmatrix"))[1] -
+                    dim(attr(RunMatrix, "modelmatrix"))[2],length(L))
+  } else {
+    degrees[is.na(degrees)] = dim(attr(RunMatrix, "modelmatrix"))[1] -
+      dim(attr(RunMatrix, "modelmatrix"))[2]
+  }
   power = c(length(L))
   for (j in 1:length(L)) {
     power[j] = calculatepower(attr(RunMatrix, "modelmatrix"), L[[j]],
-                              calcnoncentralparam(attr(RunMatrix, "modelmatrix"), L[[j]], anticoef, vinv = vinv), alpha)
+                              calcnoncentralparam(attr(RunMatrix, "modelmatrix"),
+                                                  L[[j]], anticoef, vinv = vinv),
+                              alpha, degrees[j])
   }
   return(power)
 }
